==> 1.0.0.tar.gz: lib/src/sub_types/TryParts.dart <==
import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/Types.dart';
import '../core/Identifier.dart';
import '../main/Block.dart';

/*
See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
Page 158
*/

/* <onPart> ::= <catchPart> <block> | on <typeNotVoid> <catchPart>? <block> */
class OnPart extends ParserType {
  int caseNum;
  CatchPart catchPart;
  Block block;
  TypeNotVoid typeNotVoid;

  OnPart(Lexer lexer) {
    int attempt, maxAttempts = 2;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0: _case0(lexer); break;
          case 1: _case1(lexer); break;
        }

        break;
      }
      on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts)
      throw WrongTokenType('Tried all options in OnPart: ALL FAILED');
  }
  
  /* <catchPart> <block> */
  void _case0(Lexer lexer) {
    catchPart = CatchPart(lexer);
    block = Block(lexer);
    caseNum = 0;
  }
  
  /* on <typeNotVoid> <catchPart>? <block> */
  void _case1(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'on');
    lexer.advance();

    typeNotVoid = TypeNotVoid(lexer);

    int snapShot = lexer.currentIndex;
    try {
      catchPart = CatchPart(lexer);
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    block = Block(lexer);

    caseNum = 1;
  }
  
  String toString() {
    switch (caseNum) {
      case 0: return '$catchPart $block'; break;
      case 1: return 'on $typeNotVoid ' + (catchPart != null ? '$catchPart ' : '') + '$block'; break;
    }
    return '';
  }

}

/* <catchPart> ::= catch '(' <identifier> (',' <identifier>)? ')' */
class CatchPart extends ParserType {
  Identifier identifier1;
  Identifier identifier2;

  CatchPart(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'catch');
    lexer.advance();

    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    identifier1 = Identifier(lexer);

    int snapShot = lexer.currentIndex;
    try {
      checkType(lexer, TType.COMMA);
      lexer.advance();

      identifier2 = Identifier(lexer);
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();
  }
  
  String toString() {
    return 'catch ($identifier1' +
      (identifier2 != null ? ', $identifier2' : '') + ')';
  }

}

/* <finallyPart> ::= finally <block> */
class FinallyPart extends ParserType {
  Block block;

  FinallyPart(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'finally');
    lexer.advance();

    block = Block(lexer);
  }
  
  String toString() {
    return 'finally $block';
  }

}

==> 1.1.1.tar.gz: lib/src/sub_types/TryParts.dart <==
import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/Types.dart';
import '../core/Identifier.dart';
import '../main/Block.dart';

/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 158

/// <onPart> ::= <catchPart> <block> | on <typeNotVoid> <catchPart>? <block>
class OnPart extends ParserType {
  int caseNum;
  CatchPart catchPart;
  Block block;
  TypeNotVoid typeNotVoid;

  OnPart(Lexer lexer) {
    int attempt, maxAttempts = 2;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0:
            _case0(lexer);
            break;
          case 1:
            _case1(lexer);
            break;
        }

        break;
      } on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts) {
      throw WrongTokenType('Tried all options in OnPart: ALL FAILED');
    }
  }

  /// <catchPart> <block>
  void _case0(Lexer lexer) {
    catchPart = CatchPart(lexer);
    block = Block(lexer);
    caseNum = 0;
  }

  /// on <typeNotVoid> <catchPart>? <block>
  void _case1(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'on');
    lexer.advance();

    typeNotVoid = TypeNotVoid(lexer);

    int snapShot = lexer.currentIndex;
    try {
      catchPart = CatchPart(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    block = Block(lexer);

    caseNum = 1;
  }

  String toString() {
    switch (caseNum) {
      case 0:
        return '$catchPart $block';
        break;
      case 1:
        return 'on $typeNotVoid ' +
            (catchPart != null ? '$catchPart ' : '') +
            '$block';
        break;
    }
    return '';
  }
}

/// <catchPart> ::= catch '(' <identifier> (',' <identifier>)? ')'
class CatchPart extends ParserType {
  Identifier identifier1;
  Identifier identifier2;

  CatchPart(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'catch');
    lexer.advance();

    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    identifier1 = Identifier(lexer);

    int snapShot = lexer.currentIndex;
    try {
      checkType(lexer, TType.COMMA);
      lexer.advance();

      identifier2 = Identifier(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();
  }

  String toString() {
    return 'catch ($identifier1' +
        (identifier2 != null ? ', $identifier2' : '') +
        ')';
  }
}

/// <finallyPart> ::= finally <block>
class FinallyPart extends ParserType {
  Block block;

  FinallyPart(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'finally');
    lexer.advance();

    block = Block(lexer);
  }

  String toString() {
    return 'finally $block';
  }
}
